"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var html_entities_1 = require("html-entities");
var stringify_sheet_1 = require("paperclip/lib/stringify-sheet");
var utils_1 = require("./utils");
var entities = new html_entities_1.Html5Entities();
exports.createNativeNode = function (node, protocol, map) {
    if (map === void 0) { map = new Map(); }
    // return document.createTextNode(JSON.stringify(node));
    try {
        switch (node.kind) {
            case "Text": {
                var text = createNativeTextNode(node);
                map.set(text, node.id);
                return text;
            }
            case "Element":
                return createNativeElement(node, protocol, map);
            case "StyleElement":
                return createNativeStyle(node, protocol, map);
            case "Fragment":
                return createNativeFragment(node, protocol, map);
        }
    }
    catch (e) {
        return document.createTextNode(String(e.stack));
    }
};
var createNativeTextNode = function (node) {
    return document.createTextNode(entities.decode(node.value));
};
var createNativeStyle = function (element, protocol, map) {
    // return document.createTextNode(JSON.stringify(element.sheet, null, 2));
    // return document.createTextNode(stringifyCSSSheet(element.sheet, protocol));
    var nativeElement = document.createElement("style");
    nativeElement.textContent = stringify_sheet_1.stringifyCSSSheet(element.sheet, protocol);
    map.set(nativeElement, element.id);
    return nativeElement;
};
var createNativeElement = function (element, protocol, map) {
    // return document.createTextNode(JSON.stringify(element, null, 2));
    var nativeElement = document.createElement(element.tagName);
    for (var _i = 0, _a = element.attributes; _i < _a.length; _i++) {
        var _b = _a[_i], name_1 = _b.name, value = _b.value;
        if (name_1 === "src" && protocol) {
            value = value.replace(/\w+:/, protocol);
        }
        nativeElement.setAttribute(name_1, value);
    }
    for (var _c = 0, _d = element.children; _c < _d.length; _c++) {
        var child = _d[_c];
        nativeElement.appendChild(exports.createNativeNode(child, protocol, map));
    }
    // prevent redirects & vscode from asking to redirect.
    if (element.tagName === "a") {
        nativeElement.onclick = utils_1.preventDefault;
        nativeElement.onmouseup = utils_1.preventDefault;
        nativeElement.onmousedown = utils_1.preventDefault;
    }
    map.set(nativeElement, element.id);
    return nativeElement;
};
var createNativeFragment = function (fragment, protocol, map) {
    var nativeFragment = document.createDocumentFragment();
    for (var _i = 0, _a = fragment.children; _i < _a.length; _i++) {
        var child = _a[_i];
        nativeFragment.appendChild(exports.createNativeNode(child, protocol, map));
    }
    return nativeFragment;
};
